/*
 * Copyright 2017-present Open Networking Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.onosproject.fpcagent.util.eventstream;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * SSE Server implementation.
 */
public class JettyServer implements AutoCloseable {
    private static final Logger log = LoggerFactory.getLogger(JettyServer.class);
    private static JettyServer _instance;
    private Server server;

    public static JettyServer createInstance() {
        if (_instance == null) {
            _instance = new JettyServer();
        }
        return _instance;
    }

    public static JettyServer getInstance() {
        return _instance;
    }

    /**
     * Method used to initialize and start the SSE server.
     */
    public void open() {
        ExecutorService executorService = Executors.newSingleThreadExecutor();
        executorService.submit(() -> {
            try {
                server = new Server(8070);
                ServletContextHandler context = new ServletContextHandler(ServletContextHandler.SESSIONS);
                context.setContextPath("/");

                context.addServlet(new ServletHolder(ResponseServer.class), "/response");
                context.addServlet(new ServletHolder(NotificationServer.class), "/notification");

                server.setHandler(context);

                context.addEventListener(new ConfigureService());

                try {
                    server.start();
                    log.info("Jetty-Server Started");
                    server.join();
                } catch (Exception e) {
                    log.error(ExceptionUtils.getFullStackTrace(e));
                } finally {
                    server.destroy();
                }
            } catch (Exception e) {
                log.error(ExceptionUtils.getFullStackTrace(e));
            }
        });
    }

    @Override
    public void close() {
        try {
            server.stop();
        } catch (Exception e) {
            log.error(ExceptionUtils.getFullStackTrace(e));
        }
    }
}

