/*
 * Copyright 2017-present Open Networking Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.onosproject.fpcagent.util.eventstream;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.eclipse.jetty.servlet.DefaultServlet;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.AsyncContext;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * A Http Servlet to server response streams.
 */
public class ResponseServer extends DefaultServlet {
    private static final Logger log = LoggerFactory.getLogger(ResponseServer.class);
    private static final ArrayList<String> CLIENT_URI_LIST = new ArrayList<>();

    /**
     * Method for stream initialization.
     *
     * @param clientUri - Client Uri
     * @param request   - The servlet request object
     * @param response  - The servlet Response Object
     */
    private void init(String clientUri, HttpServletRequest request, HttpServletResponse response) {
        log.info("Response Stream Inititated");
        try {
            HttpSession session = request.getSession();
            session.setMaxInactiveInterval(72 * 60 * 60);
            RequestServer client = new RequestServer();
            client.connectToClient(clientUri);
            response.setHeader("Content-Type", "text/event-stream");
            response.setHeader("Cache-Control", "no-cache, no-store");
            response.setHeader("Connection", "keep-alive");
            AsyncContext asyncContext = request.startAsync(request, response);
            asyncContext.setTimeout(72 * 60 * 60 * 1000);
            asyncContext.getResponse().setBufferSize(1200);
            try {
                asyncContext.getResponse().flushBuffer();
            } catch (IOException e) {
                log.error(ExceptionUtils.getFullStackTrace(e));
            }
            ServletContext servletContext = request.getServletContext();
            Map<String, AsyncContext> responseStreams =
                    (ConcurrentHashMap<String, AsyncContext>) servletContext.getAttribute("responseStreams");
            responseStreams.put(clientUri, asyncContext);
        } catch (Exception e) {
            log.error(ExceptionUtils.getFullStackTrace(e));
        }
    }

    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response) {
        try {
            String clientUri = null;
            StringBuffer jsonStringBuilder = new StringBuffer();
            String line = null;
            try {
                BufferedReader br = request.getReader();
                while ((line = br.readLine()) != null) {
                    jsonStringBuilder.append(line);
                }
            } catch (Exception e) {
                log.error(ExceptionUtils.getFullStackTrace(e));
            }

            try {
                if (jsonStringBuilder.length() > 0) {
                    JSONObject jsonObj = new JSONObject(jsonStringBuilder.toString());
                    clientUri = jsonObj.getString("client-uri");
                    if (!CLIENT_URI_LIST.contains(clientUri)) {
                        init(clientUri, request, response);
                    }
                    jsonStringBuilder.setLength(0);
                }
            } catch (JSONException e) {
                log.error(ExceptionUtils.getFullStackTrace(e));
            }
        } catch (Exception e) {
            log.error(ExceptionUtils.getFullStackTrace(e));
        }

    }
}
